#include <iostream>
#include <string>
using namespace std;
#include "Car.h"
#include "Passenger.h"

Car::Car()
  : m_make("Toyota"), m_model("Corolla"), m_seats(5), m_seatsTaken(0), m_mpg(35)
{ 
  m_passengersPtr = new Passenger[ m_seats ];
}


Car::Car(string make, string model, int seats, int mpg)
  : m_make(make), m_model(model), m_seats(seats), m_seatsTaken(0), m_mpg(mpg)
{ 
  m_passengersPtr = new Passenger[ m_seats ];
}

Car::Car(const Car& car)
  : m_make(car.m_make), m_model(car.m_model), m_seats(car.m_seats),
    m_seatsTaken(car.m_seatsTaken)
{
  m_passengersPtr = new Passenger[ car.m_seats ];
  
  for (int i = 0; i < m_seatsTaken; i++)
    m_passengersPtr[i] = car.m_passengersPtr[i];
}

string Car::GetMake() const
{
  return m_make;
}

string Car::GetModel() const
{
  return m_model;
}

int Car::GetSeats() const 
{
  return m_seats;
}

int Car::GetSeatsTaken() const
{
  return m_seatsTaken;
}

int Car::GetMpg() const
{
  return m_mpg;
}

Passenger* Car::GetPassengers() const
{
  return m_passengersPtr;
}

void Car::SetMake(string make)
{
  m_make = make;
}

void Car::SetModel(string model)
{
  m_model = model;
}

void Car::SetSeats(int numSeats)
{
  m_seats = numSeats;
}

void Car::SetMpg(int mpg)
{
  m_mpg = mpg;
}

void Car::AddPassenger(Passenger passenger)
{
  if (m_seatsTaken < m_seats)
    m_passengersPtr[ m_seatsTaken++ ] = passenger;
  else
    cerr << "The car is already full!" << endl;
}

Car::~Car()
{
  delete [] m_passengersPtr;
}


Car& Car::operator =(const Car& car)
{
  if (this != &car) {

    m_make = car.m_make;
    m_model = car.m_model;
    m_seatsTaken = car.m_seatsTaken;

    if (m_seats != car.m_seats) {

      delete [] m_passengersPtr;
      m_passengersPtr = new Passenger[ car.m_seats ];
    
      m_seats = car.m_seats;

    }
    for (int i = 0; i < m_seatsTaken; i++)
      m_passengersPtr[i] = car.m_passengersPtr[i];

  }

  // 'this' is a pointer to the current object
  return *this;

}

ostream& operator <<(ostream& sout, const Car& car)
{
  sout << car.m_make << " " << car.m_model 
       << " (" << car.m_mpg << " mpg)" << endl;
  for (int i = 0; i < car.m_seatsTaken; i++)
    sout << car.m_passengersPtr[i] << endl;

  return sout;
}

  
